@@ -2,7 +2,7 @@
 
 const object = {};
 const hasOwnProperty = object.hasOwnProperty;
-const forOwn = function(object, callback) {
+const forOwn = (object, callback) => {
 	for (const key in object) {
 		if (hasOwnProperty.call(object, key)) {
 			callback(key, object[key]);
@@ -10,17 +10,17 @@ const forOwn = function(object, callback) {
 	}
 };
 
-const extend = function(destination, source) {
+const extend = (destination, source) => {
 	if (!source) {
 		return destination;
 	}
-	forOwn(source, function(key, value) {
+	forOwn(source, (key, value) => {
 		destination[key] = value;
 	});
 	return destination;
 };
 
-const forEach = function(array, callback) {
+const forEach = (array, callback) => {
 	const length = array.length;
 	let index = -1;
 	while (++index < length) {
@@ -28,28 +28,38 @@ const forEach = function(array, callback) {
 	}
 };
 
+const fourHexEscape = (hex) => {
+	return '\\u' + ('0000' + hex).slice(-4);
+}
+
+const hexadecimal = (code, lowercase) => {
+	let hexadecimal = code.toString(16);
+	if (lowercase) return hexadecimal;
+	return hexadecimal.toUpperCase();
+};
+
 const toString = object.toString;
 const isArray = Array.isArray;
 const isBuffer = Buffer.isBuffer;
-const isObject = function(value) {
+const isObject = (value) => {
 	// This is a very simple check, but it’s good enough for what we need.
 	return toString.call(value) == '[object Object]';
 };
-const isString = function(value) {
+const isString = (value) => {
 	return typeof value == 'string' ||
 		toString.call(value) == '[object String]';
 };
-const isNumber = function(value) {
+const isNumber = (value) => {
 	return typeof value == 'number' ||
 		toString.call(value) == '[object Number]';
 };
-const isFunction = function(value) {
+const isFunction = (value) => {
 	return typeof value == 'function';
 };
-const isMap = function(value) {
+const isMap = (value) => {
 	return toString.call(value) == '[object Map]';
 };
-const isSet = function(value) {
+const isSet = (value) => {
 	return toString.call(value) == '[object Set]';
 };
 
@@ -57,8 +67,6 @@ const isSet = function(value) {
 
 // https://mathiasbynens.be/notes/javascript-escapes#single
 const singleEscapes = {
-	'"': '\\"',
-	'\'': '\\\'',
 	'\\': '\\\\',
 	'\b': '\\b',
 	'\f': '\\f',
@@ -68,13 +76,16 @@ const singleEscapes = {
 	// `\v` is omitted intentionally, because in IE < 9, '\v' == 'v'.
 	// '\v': '\\x0B'
 };
-const regexSingleEscape = /["'\\\b\f\n\r\t]/;
+const regexSingleEscape = /[\\\b\f\n\r\t]/;
 
 const regexDigit = /[0-9]/;
-const regexWhitelist = /[ !#-&\(-\[\]-~]/;
+const regexWhitespace = /[\xA0\u1680\u2000-\u200A\u2028\u2029\u202F\u205F\u3000]/;
 
-const jsesc = function(argument, options) {
-	const increaseIndentation = function() {
+const escapeEverythingRegex = /([\uD800-\uDBFF][\uDC00-\uDFFF])|([\uD800-\uDFFF])|(['"`])|[^]/g;
+const escapeNonAsciiRegex = /([\uD800-\uDBFF][\uDC00-\uDFFF])|([\uD800-\uDFFF])|(['"`])|[^ !#-&\(-\[\]-_a-~]/g;
+
+const jsesc = (argument, options) => {
+	const increaseIndentation = () => {
 		oldIndent = indent;
 		++options.indentLevel;
 		indent = options.indent.repeat(options.indentLevel)
@@ -102,10 +113,19 @@ const jsesc = function(argument, options) {
 		defaults.wrap = true;
 	}
 	options = extend(defaults, options);
-	if (options.quotes != 'single' && options.quotes != 'double') {
+	if (
+		options.quotes != 'single' &&
+		options.quotes != 'double' &&
+		options.quotes != 'backtick'
+	) {
 		options.quotes = 'single';
 	}
-	const quote = options.quotes == 'double' ? '"' : '\'';
+	const quote = options.quotes == 'double' ?
+		'"' :
+		(options.quotes == 'backtick' ?
+			'`' :
+			'\''
+		);
 	const compact = options.compact;
 	const lowercaseHex = options.lowercaseHex;
 	let indent = options.indent.repeat(options.indentLevel);
@@ -143,9 +163,9 @@ const jsesc = function(argument, options) {
 		}
 		if (isBuffer(argument)) {
 			if (argument.length == 0) {
-				return 'Buffer()';
+				return 'Buffer.from([])';
 			}
-			return 'Buffer(' + jsesc(Array.from(argument), options) + ')';
+			return 'Buffer.from(' + jsesc(Array.from(argument), options) + ')';
 		}
 		if (isArray(argument)) {
 			result = [];
@@ -157,7 +177,7 @@ const jsesc = function(argument, options) {
 			if (!inline2) {
 				increaseIndentation();
 			}
-			forEach(argument, function(value) {
+			forEach(argument, (value) => {
 				isEmpty = false;
 				if (inline2) {
 					options.__inline2__ = false;
@@ -208,7 +228,7 @@ const jsesc = function(argument, options) {
 			result = [];
 			options.wrap = true;
 			increaseIndentation();
-			forOwn(argument, function(key, value) {
+			forOwn(argument, (key, value) => {
 				isEmpty = false;
 				result.push(
 					(compact ? '' : indent) +
@@ -225,89 +245,72 @@ const jsesc = function(argument, options) {
 		}
 	}
 
-	const string = argument;
-	// Loop over each code unit in the string and escape it
-	let index = -1;
-	const length = string.length;
-	result = '';
-	while (++index < length) {
-		const character = string.charAt(index);
-		if (options.es6) {
-			const first = string.charCodeAt(index);
-			if ( // check if it’s the start of a surrogate pair
-				first >= 0xD800 && first <= 0xDBFF && // high surrogate
-				length > index + 1 // there is a next code unit
-			) {
-				const second = string.charCodeAt(index + 1);
-				if (second >= 0xDC00 && second <= 0xDFFF) { // low surrogate
-					// https://mathiasbynens.be/notes/javascript-encoding#surrogate-formulae
-					const codePoint = (first - 0xD800) * 0x400 + second - 0xDC00 + 0x10000;
-					let hexadecimal = codePoint.toString(16);
-					if (!lowercaseHex) {
-						hexadecimal = hexadecimal.toUpperCase();
-					}
-					result += '\\u{' + hexadecimal + '}';
-					++index;
-					continue;
-				}
+	const regex = options.escapeEverything ? escapeEverythingRegex : escapeNonAsciiRegex;
+	result = argument.replace(regex, (char, pair, lone, quoteChar, index, string) => {
+		if (pair) {
+			if (options.minimal) return pair;
+			const first = pair.charCodeAt(0);
+			const second = pair.charCodeAt(1);
+			if (options.es6) {
+				// https://mathiasbynens.be/notes/javascript-encoding#surrogate-formulae
+				const codePoint = (first - 0xD800) * 0x400 + second - 0xDC00 + 0x10000;
+				const hex = hexadecimal(codePoint, lowercaseHex);
+				return '\\u{' + hex + '}';
 			}
+			return fourHexEscape(hexadecimal(first, lowercaseHex)) + fourHexEscape(hexadecimal(second, lowercaseHex));
 		}
-		if (!options.escapeEverything) {
-			if (regexWhitelist.test(character)) {
-				// It’s a printable ASCII character that is not `"`, `'` or `\`,
-				// so don’t escape it.
-				result += character;
-				continue;
-			}
-			if (character == '"') {
-				result += quote == character ? '\\"' : character;
-				continue;
-			}
-			if (character == '\'') {
-				result += quote == character ? '\\\'' : character;
-				continue;
-			}
+
+		if (lone) {
+			return fourHexEscape(hexadecimal(lone.charCodeAt(0), lowercaseHex));
 		}
+
 		if (
-			character == '\0' &&
+			char == '\0' &&
 			!json &&
 			!regexDigit.test(string.charAt(index + 1))
 		) {
-			result += '\\0';
-			continue;
+			return '\\0';
 		}
-		if (regexSingleEscape.test(character)) {
+
+		if (quoteChar) {
+			if (quoteChar == quote || options.escapeEverything) {
+				return '\\' + quoteChar;
+			}
+			return quoteChar;
+		}
+
+		if (regexSingleEscape.test(char)) {
 			// no need for a `hasOwnProperty` check here
-			result += singleEscapes[character];
-			continue;
+			return singleEscapes[char];
 		}
-		const charCode = character.charCodeAt(0);
-		if (options.minimal && charCode != 0x2028 && charCode != 0x2029) {
-			result += character;
-			continue;
+
+		if (options.minimal && !regexWhitespace.test(char)) {
+			return char;
 		}
-		let hexadecimal = charCode.toString(16);
-		if (!lowercaseHex) {
-			hexadecimal = hexadecimal.toUpperCase();
+
+		const hex = hexadecimal(char.charCodeAt(0), lowercaseHex);
+		if (json || hex.length > 2) {
+			return fourHexEscape(hex);
 		}
-		const longhand = hexadecimal.length > 2 || json;
-		const escaped = '\\' + (longhand ? 'u' : 'x') +
-			('0000' + hexadecimal).slice(longhand ? -4 : -2);
-		result += escaped;
-		continue;
+
+		return '\\x' + ('00' + hex).slice(-2);
+	});
+
+	if (quote == '`') {
+		result = result.replace(/\$\{/g, '\\${');
+	}
+	if (options.isScriptContext) {
+		// https://mathiasbynens.be/notes/etago
+		result = result
+			.replace(/<\/(script|style)/gi, '<\\/$1')
+			.replace(/<!--/g, json ? '\\u003C!--' : '\\x3C!--');
 	}
 	if (options.wrap) {
 		result = quote + result + quote;
 	}
-	if (options.isScriptContext) {
-		// https://mathiasbynens.be/notes/etago
-		return result
-			.replace(/<\/(script|style)/gi, '<\\/$1')
-			.replace(/<!--/g, json ? '\\u003C!--' : '\\x3C!--');
-	}
 	return result;
 };
 
-jsesc.version = '2.4.0';
+jsesc.version = '3.0.1';
 
 module.exports = jsesc;
